@@ -22,7 +22,7 @@ our @ISA = qw(Exporter);
 # If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
 # will save memory.
 our %EXPORT_TAGS = (
-	'all' => [ qw(upload) ],
+	'all' => [ qw(upload make_upload_request upload_request) ],
 );
 
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
@@ -30,7 +30,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw(
 );
 
-our $VERSION = do { my @r = (q$Revision: 1.6 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+our $VERSION = do { my @r = (q$Revision: 1.7 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 # Preloaded methods go here.
 
@@ -57,45 +57,6 @@ sub uploader_tag($$) {
 	return undef;
 }
 
-sub upload {
-	my $ua = shift;
-	my %args = @_;
-
-	# these are the only things _required_ by the uploader.
-	return undef unless $args{'photo'} and -f $args{'photo'};
-	return undef unless $args{'email'};
-	return undef unless $args{'password'};
-
-	my $photo = $args{'photo'};
-	my $uri = $args{'uri'} || 'http://www.flickr.com/tools/uploader_go.gne';
-
-	# strip these from the hash so we can just drop it into the request
-	delete $args{$_} for(qw(photo uri));
-
-	my $req = POST $uri,
-		'Content_Type' => 'form-data',
-		'Content' => [
-			'photo' => [ $photo ],
-			%args,
-		];
-
-	my $res = $ua->request( $req );
-
-	my $tree = XML::Parser::Lite::Tree::instance()->parse($res->content());
-
-	my $photoid = uploader_tag($tree, 'photoid');
-	unless( defined $photoid ) {
-		my $err = uploader_tag($tree, 'verbose');
-		print STDERR "upload failed: ", ($err || $res->content()), "\n";
-		return undef;
-	}
-
-	return $photoid;
-}
-
-1;
-__END__
-
 =head1 NAME
 
 Flickr::Upload - Upload images to L<flickr.com>
@@ -147,6 +108,126 @@ C<undef> on failure. According to the API documentation, after an upload the
 user should be directed to the page
 L<http://www.flickr.com/tools/uploader_edit.gne?ids=$photoid>.
 
+=cut
+#'
+
+sub upload($%) {
+	my $ua = shift;
+	die "expecting a LWP::UserAgent" unless $ua->isa('LWP::UserAgent');
+	my %args = @_;
+
+	# these are the only things _required_ by the uploader.
+	return undef unless $args{'photo'} and -f $args{'photo'};
+	return undef unless $args{'email'};
+	return undef unless $args{'password'};
+
+	# create a request object and execute it
+	my $req = make_upload_request( %args );
+	return undef unless defined $req;
+
+	return upload_request( $ua, $req );
+}
+
+=head2 make_upload_request
+
+	my $req = make_upload_request(
+		'email' => 'self@example.com',
+		'password' => 'pr1vat3',
+		'tags' => 'me myself eye',
+		'is_public' => 1,
+		'is_friend' => 1,
+		'is_family' => 1
+	);
+	$req->header( 'X-Greetz' => 'hi cal' );
+	my $resp = $ua->request( $req );
+
+Creates an L<HTTP::Request> object loaded with all the flick upload
+parameters.
+
+Takes all the same parameters as L<upload>, except that the photo argument
+isn't required. This in intended so that the caller can include it by
+messing directly with the HTTP content (via C<$DYNAMIC_FILE_UPLOAD> or
+the L<HTTP::Message> class, among other things).
+
+Returns a standard L<HTTP::Response> POST object. The caller can manually
+do the upload or just call the L<upload_request> function.
+
+=cut
+#'
+
+sub make_upload_request(%) {
+	my %args = @_;
+
+	# these are the only things _required_ by the uploader.
+	return undef unless $args{'email'};
+	return undef unless $args{'password'};
+
+	my $uri = $args{'uri'} || 'http://www.flickr.com/tools/uploader_go.gne';
+
+	# passed in separately, so remove from the hash
+	delete $args{uri};
+
+	# we don't do async
+	$args{async} = 0;
+
+	if( exists $args{photo} and ref $args{photo} ne "ARRAY" ) {
+		# unlikely that the caller would set up the photo as an array,
+		# but...
+		$args{photo} = [ $args{photo} ];
+	}
+
+	return POST $uri,
+		'Content_Type' => 'form-data',
+		'Content' => [ %args ];
+}
+
+=head2 upload_request
+
+	my $photoid = upload_request( $ua, $request );
+
+Taking L<LWP::UserAgent> and L<HTTP::Request> objects as arguments, this
+executes the request and processes the result as a flickr upload. It's
+assumed that the request looks a lot like something created with
+L<make_upload_request>.
+
+Returns the resulting identifier of the uploaded photo on success,
+C<undef> on failure. According to the API documentation, after an upload the
+user should be directed to the page
+L<http://www.flickr.com/tools/uploader_edit.gne?ids=$photoid>.
+
+=cut
+#'
+
+sub upload_request($$) {
+	my $ua = shift;
+	die "expecting a LWP::UserAgent" unless $ua->isa('LWP::UserAgent');
+	my $req = shift;
+	die "expecting a HTTP::Request" unless $req->isa('HTTP::Request');
+
+	my $res = $ua->request( $req );
+
+	my $tree = XML::Parser::Lite::Tree::instance()->parse($res->content());
+
+	my $photoid = uploader_tag($tree, 'photoid');
+	unless( defined $photoid ) {
+		my $err = uploader_tag($tree, 'verbose');
+		print STDERR "upload failed: ", ($err || $res->content()), "\n";
+		return undef;
+	}
+
+	return $photoid;
+}
+
+1;
+__END__
+
+=head1 BUGS
+
+Asynchronous uploading isn't supported. Async, quite frankly, doesn't
+make much sense in this API. Anyone who can write a perl script to get
+an image upload going but is unable to keep track of whether or not it
+finished without polling a web server has got major problems.
+
 =head1 SEE ALSO
 
 L<http://flickr.com/services/api/>.
@@ -157,15 +238,10 @@ Christophe Beauregard, L<cpb@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This module is not an official Flickr.com (or Ludicorp) service. I'm sure
-if they knew about it they could suggest some additional words saying just
-how little they're responsible for anything that might go wrong with this
-code.
+This module is not an official Flickr.com (or Ludicorp) service.
 
 Copyright (C) 2004 by Christophe Beauregard
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.3 or,
 at your option, any later version of Perl 5 you may have available.
-
-=cut
